#!/bin/bash

# Filename: smtt-make-checksums.sh
# Synopsis: Script that generates the checksums for the filesystem
#           files and the boot config file, and store all of them
#           into each of the initrd files present in the live/
#           directory on the USB.
# Parameters: It takes as parameter the PATH/TO/USB
# Author: Luis Dalo
# Version: 0.0.1
# Last Edited: 9 Nov 2017

set -e

###############
#  VARIABLES  #
###############

DIGEST="sha256sum"
HELP="\nUsage: ${0}  [PATH_TO_USB]\n
Where:\n
- PATH_TO_USB: Absolute path of the media where the USB is mounted.\n"
CHECKSUM_FILENAME=/tmp/live-boot-${DIGEST^^}.txt

##########################
#  VERIFYING PARAMETERS  #
##########################

if [ $# -ne 1 ]
then
    echo "E: Wrong usage"
    echo -e ${HELP}
    exit 1
fi

if [ ! -d $1 ]
then
    echo "E: USB path $1 not a directory"
    echo -e ${HELP}
    exit 1
else
    # Assigning USB path
    USB_PATH=$1
fi

if [ ! -d ${USB_PATH}/live ]
then
    echo "E: No live/ directory inside ${USB_PATH}"
    echo -e ${HELP}
    exit 1
fi

if [ ! -d ${USB_PATH}/boot ]
then
    echo "E: No boot/ directory inside ${USB_PATH}"
    echo -e ${HELP}
    exit 1
fi

if [ $(find ${USB_PATH}/live/ -type f -name "initrd*" | wc -l) -eq 0 ]
then
    echo "E: No initrd files found inside ${USB_PATH}/live/ directory"
    echo -e ${HELP}
    exit 1
fi

#######################
#  DYNAMIC VARIABLES  #
#######################

INITRD_DIR=$(mktemp --directory --tmpdir=/tmp initrd.XXXXX -u)

###########
#  BEGIN  #
###########

# changing directory
cd ${USB_PATH}

echo "I: Calculating ${DIGEST^^} of the contents of the live/ directory"
find live/ -type f -not -name "initrd*" -exec ${DIGEST} {} \; > ${CHECKSUM_FILENAME}

echo "I: Calculating ${DIGEST^^} of the most relevant files in the boot/ directory"
find boot/ -type f \( -name "host_*" -o -name "grub.cfg" \) -exec ${DIGEST} {} \; >> ${CHECKSUM_FILENAME}

# going back to current directory
cd - > /dev/null

initrd_list=$(find ${USB_PATH}/live/ -type f -name "initrd*")

for initrd in ${initrd_list}
do
    test -d ${INITRD_DIR} && rm -rf ${INITRD_DIR}
    mkdir -p ${INITRD_DIR}
    echo "I: Extracting $(basename ${initrd})"
    cd ${INITRD_DIR}
    gunzip < ${initrd} | cpio -i || { echo "E: Unable to read file $(basename ${initrd})" ; exit 1; }
    echo "I: Copying checksums file into $(basename ${initrd})"
    cp ${CHECKSUM_FILENAME} .
    echo "I: Repacking $(basename ${initrd})"
    find | cpio -H newc -o | gzip -9 > ${initrd} || { echo "E: Unable to write file ${initrd}" ; exit 1; }
    cd - > /dev/null
done

##############
#  CLEAN UP  #
##############

rm -rf ${INITRD_DIR}
rm -rf ${CHECKSUM_FILENAME}

echo "I: Checksums DONE!"

#########
#  END  #
#########
