# Gettext support: Create/Update pot file and
#
# To use: INCLUDE(Gettext)
#
# Most of the gettext support code is from FindGettext.cmake of cmake,
# but it is included here because:
#
# 1. Some system like RHEL5 does not have FindGettext.cmake
# 2. Bug of GETTEXT_CREATE_TRANSLATIONS make it unable to be include in 'All'
# 3. It does not support xgettext
#
#===================================================================
# Constants:
#  XGETTEXT_OPTIONS_DEFAULT: Default xgettext option:
#===================================================================
# Variables:
#  XGETTEXT_OPTIONS: Options pass to xgettext
#   Default:  XGETTEXT_OPTIONS_DEFAULT
#  GETTEXT_MSGMERGE_EXECUTABLE: the full path to the msgmerge tool.
#  GETTEXT_MSGFMT_EXECUTABLE: the full path to the msgfmt tool.
#  GETTEXT_FOUND: True if gettext has been found.
#  XGETTEXT_EXECUTABLE: the full path to the xgettext.
#  XGETTEXT_FOUND: True if xgettext has been found.
#
#===================================================================
# Macros:
# GETTEXT_CREATE_POT(potFile
# [OPTION xgettext_options]
# SRC list_of_source_file_that_contains_msgid
# )
#
# Generate .pot file.
# OPTION xgettext_options: Override XGETTEXT_OPTIONS
#
# * Produced targets: pot_file
#
#-------------------------------------------------------------------
# GETTEXT_CREATE_TRANSLATIONS ( outputFile [ALL] locale1 ... localeN
#  [COMMENT comment] )
#
#  This will create a target "translations" which will convert the
#  given input po files into the binary output mo file. If the
#  ALL option is used, the translations will also be created when
#  building the default target.
#
# * Produced targets: translations
#-------------------------------------------------------------------

IF (COMMAND FIND_HOST_PROGRAM)
  FIND_HOST_PROGRAM (GETTEXT_MSGMERGE_EXECUTABLE msgmerge)
  FIND_HOST_PROGRAM (GETTEXT_MSGFMT_EXECUTABLE msgfmt)
  FIND_HOST_PROGRAM (GETTEXT_MSGCAT_EXECUTABLE msgcat)
  FIND_HOST_PROGRAM (XGETTEXT_EXECUTABLE xgettext)
  FIND_HOST_PROGRAM (GETTEXT_MSGINIT_EXECUTABLE msginit)
ELSE ()
  FIND_PROGRAM (GETTEXT_MSGMERGE_EXECUTABLE msgmerge)
  FIND_PROGRAM (GETTEXT_MSGFMT_EXECUTABLE msgfmt)
  FIND_PROGRAM (GETTEXT_MSGCAT_EXECUTABLE msgcat)
  FIND_PROGRAM (XGETTEXT_EXECUTABLE xgettext)
  FIND_PROGRAM (GETTEXT_MSGINIT_EXECUTABLE msginit)
ENDIF ()

SET (XGETTEXT_OPTIONS_DEFAULT
  --keyword=_ --keyword=N_ --keyword=C_:1c,2 --keyword=NC_:1c,2 -s
  --omit-header --from-code=UTF-8 --escape --add-comments="/" --package-name=${PROJECT_NAME} --package-version=${VERSION}
  )

IF (GETTEXT_MSGMERGE_EXECUTABLE AND GETTEXT_MSGFMT_EXECUTABLE AND GETTEXT_MSGCAT_EXECUTABLE)
  SET (GETTEXT_FOUND TRUE)
ELSE ()
  SET (GETTEXT_FOUND FALSE)

  IF (GetText_REQUIRED)
    MESSAGE (FATAL_ERROR "GetText not found")
  ENDIF ()
ENDIF ()

IF (XGETTEXT_EXECUTABLE)
  SET (XGETTEXT_FOUND TRUE)
ELSE ()
  MESSAGE (STATUS "xgettext not found.")
  SET (XGETTTEXT_FOUND FALSE)
ENDIF ()

IF (NOT DEFINED XGETTEXT_OPTIONS)
  SET (XGETTEXT_OPTIONS ${XGETTEXT_OPTIONS_DEFAULT})
ENDIF ()

function(_GETTEXT_GET_UNIQUE_TARGET_NAME _name _unique_name)
   set(propertyName "_GETTEXT_UNIQUE_COUNTER_${_name}")
   get_property(currentCounter GLOBAL PROPERTY "${propertyName}")
   if(NOT currentCounter)
      set(currentCounter 1)
   endif()
   set(${_unique_name} "${_name}_${currentCounter}" PARENT_SCOPE)
   math(EXPR currentCounter "${currentCounter} + 1")
   set_property(GLOBAL PROPERTY ${propertyName} ${currentCounter} )
endfunction()

#
# Create Pot macro.
#
MACRO (GETTEXT_CREATE_POT _potFile _pot_options)
  GET_FILENAME_COMPONENT (_pot_directory "${_potFile}" DIRECTORY)
  FILE (MAKE_DIRECTORY "${_pot_directory}")

  SET (_xgettext_options_list)
  SET (_src_list)
  SET (_src_list_abs)
  SET (_glade_list)
  SET (_glade_list_abs)
  SET (_stage "SRC")

  FOREACH (_pot_option ${_pot_options} ${ARGN})
    IF (_pot_option STREQUAL "OPTION")
      SET (_stage "OPTION")
    ELSEIF (_pot_option STREQUAL "SRC")
      SET (_stage "SRC")
    ELSEIF (_pot_option STREQUAL "GLADE")
      SET (_stage "GLADE")
    ELSE ()
      IF (_stage STREQUAL "OPTION")
        SET (_xgettext_options_list ${_xgettext_options_list} ${_pot_option})
      ELSEIF (_stage STREQUAL "GLADE")
        FILE (RELATIVE_PATH _relFile ${CMAKE_CURRENT_SOURCE_DIR} ${_pot_option})
        GET_FILENAME_COMPONENT (_absFile ${_pot_option} ABSOLUTE)
        SET (_glade_list ${_glade_list} ${_relFile})
        SET (_glade_list_abs ${_glade_list_abs} ${_absFile})
      ELSEIF (_stage STREQUAL "SRC")
        FILE (RELATIVE_PATH _relFile ${CMAKE_CURRENT_SOURCE_DIR} ${_pot_option})
        GET_FILENAME_COMPONENT (_absFile ${_pot_option} ABSOLUTE)
        SET (_src_list ${_src_list} ${_relFile})
        SET (_src_list_abs ${_src_list_abs} ${_absFile})
      ENDIF ()
    ENDIF ()
  ENDFOREACH ()

 IF ()
  SET (_xgettext_options ${_xgettext_options_list})
 ELSE ()
  SET (_xgettext_options ${XGETTEXT_OPTIONS})
 ENDIF ()

#MESSAGE ( STATUS "XGETTEXT_EXECUTABLE ${_xgettext_options_list} -o ${_potFile} {_src_list}" )

 EXECUTE_PROCESS (
  WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
  COMMAND ${XGETTEXT_EXECUTABLE} ${_xgettext_options_list} -o ${_potFile} ${_src_list}
 )
ENDMACRO ()



FUNCTION (GETTEXT_PROCESS_POT_FILE _potFile)
  SET (_gmoFiles)
  SET (_options ALL)
  SET (_oneValueArgs DATA_BUILD_DIR)
  SET (_multiValueArgs LANGUAGES)

  CMAKE_PARSE_ARGUMENTS (_parsedArguments "${_options}" "${_oneValueArgs}" "${_multiValueArgs}" ${ARGN})

  GET_FILENAME_COMPONENT (_potName ${_potFile} NAME)
  STRING (REGEX REPLACE "^(.+)(\\.[^.]+)$" "\\1" _potBasename ${_potName})
  GET_FILENAME_COMPONENT (_absPotFile ${_potFile} ABSOLUTE)

  IF (EXISTS ${_potFile})
    GET_FILENAME_COMPONENT (_potPath ${_potFile} DIRECTORY)
    FOREACH (_lang ${_parsedArguments_LANGUAGES})
      SET (_poFile  "${_potPath}/${_lang}.po")
      SET (_gmoFile "${_parsedArguments_DATA_BUILD_DIR}/${_lang}/LC_MESSAGES/${_potBasename}.mo")

      IF (NOT EXISTS ${_poFile} )
        #MESSAGE ( STATUS "GETTEXT_MSGINIT_EXECUTABLE -i ${_potFile} -o ${_poFile}" )
        EXECUTE_PROCESS (
          WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
          COMMAND ${GETTEXT_MSGINIT_EXECUTABLE} --no-translator -i ${_potFile} -o ${_poFile}
          )
      ENDIF ()

      #MESSAGE ( STATUS "GETTEXT_MSGMERGE_EXECUTABLE --quiet --update --backup=none -s ${_poFile} ${_absPotFile}" )
      EXECUTE_PROCESS (
        WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
        COMMAND ${GETTEXT_MSGMERGE_EXECUTABLE} --quiet --update --backup=none -s ${_poFile} ${_absPotFile}
        )

      GET_FILENAME_COMPONENT (modir ${_gmoFile} DIRECTORY)

      IF (NOT EXISTS ${modir} )
        #MESSAGE ( STATUS "MAKE DIR ${modir}" )
        FILE (MAKE_DIRECTORY ${modir})
      ENDIF ()

      #MESSAGE ( STATUS "GETTEXT_MSGFMT_EXECUTABLE -o ${_gmoFile} ${_poFile}" )
      EXECUTE_PROCESS (
        WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
        COMMAND ${GETTEXT_MSGFMT_EXECUTABLE} -o ${_gmoFile} ${_poFile}
        )
    ENDFOREACH ()
  ELSE ()
    MESSAGE (FATAL_ERROR "POT file ${_potFile} not found!")
  ENDIF ()
ENDFUNCTION ()
