/**
 * @file	MemoryPathController.cpp
 * @brief	Implements the memory path controller class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#include "MemoryPathController.h"
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <sys/mount.h>
#include <System/Utils/StringUtils.hxx>
#include <iostream>

using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

MemoryPathController * MemoryPathController::instance = NULL;

/**
 * @brief  Removes a path from the path list.
 * @author manuel.arguelles
 * @date   Sep  7, 2017
 * @param  pathList The vector with the pathList.
 * @param  path     The entry to remove.
 */
static void removeFromPathList(std::vector<std::string>& pathList, const std::string& path)
{
    std::vector<std::string>::iterator it;

    for (it = pathList.begin(); it != pathList.end(); ++it)
    {
        if (*it == path)
        {
            pathList.erase(it);
            break;
        }
    }
}

MemoryPathController::MemoryPathController()
{
    ConfigurationElectionFilesPath & configElectionFiles ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath() );
    ConfigurationBasePath configBasePath = configElectionFiles.getConfigBasePath();
    rule = configBasePath.getRulesBasePath();
    liveFileSystemBasePath = configBasePath.getLiveFileSystemBasePath();
}

MemoryPathController::~MemoryPathController()
{
}

std::string & MemoryPathController::getStartupMemoryBasePath()
{
    return domMemoryBasePath; ///< path of the startup memory
}

std::string & MemoryPathController::getLiveFileSystemBasePath()
{
    return liveFileSystemBasePath; ///< path of the live folder
}

std::string & MemoryPathController::getDomMemoryBasePath()
{
    return domMemoryBasePath; ///< path of the DOM memory
}

std::string & MemoryPathController::getFlashMemoryBasePath()
{
    return flashMemoryBasePath;///< path of the Flash Memory
}

std::string & MemoryPathController::getBackupMemoryBasePath()
{
    return backupMemoryBasePath;
}

void MemoryPathController::updatePaths()
{
    domMemoryBasePath.clear();
    flashMemoryBasePath.clear();
    backupMemoryBasePath.clear();

    std::vector < std::string >::iterator it;
    int count = 0;

    for ( it = pathList.begin(); it != pathList.end(); ++it )
    {
        std::string path = *it;

        if ( Directory::getInstance()->DirectoryExists ( path ) )
        {
            switch ( count )
            {
                default:
                case 0:
                    domMemoryBasePath = path;
                    break;

                case 1:
                    flashMemoryBasePath = path;
                    break;

                case 2:
                    backupMemoryBasePath = path;
                    break;
            }

            count++;
        }
    }

    switch ( rule )
    {
        default:
        case RulesBasePathType::CONFIGURATION_PM_RULE:
            break;

        case RulesBasePathType::CONFIGURATION_VM_RULE:
            backupMemoryBasePath = domMemoryBasePath;
            domMemoryBasePath = liveFileSystemBasePath;
            break;
    }

    std::cout << "DomMemoryBasePath : " << domMemoryBasePath << std::endl;
    std::cout << "FlashMemoryBasePath : " << flashMemoryBasePath << std::endl;
}

RulesBasePathType & MemoryPathController::getRule()
{
    return rule;
}

bool MemoryPathController::isAllDevices()
{
    bool result = true;

    switch ( rule )
    {
        default:
        case RulesBasePathType::CONFIGURATION_PM_RULE:
            result = Directory::getInstance()->DirectoryExists ( domMemoryBasePath ) && Directory::getInstance()->DirectoryExists ( flashMemoryBasePath );
            break;

        case RulesBasePathType::CONFIGURATION_VM_RULE:
            result = Directory::getInstance()->DirectoryExists ( liveFileSystemBasePath );
            break;
    }

    return result;
}

MemoryPathController * MemoryPathController::getInstance()
{
    if ( MemoryPathController::instance == NULL )
    {
        MemoryPathController::instance = new MemoryPathController();
    }

    return MemoryPathController::instance;
}

void MemoryPathController::addPath ( std::string path )
{
    removeFromPathList(pathList, path);
    pathList.push_back(path);
    updatePaths();
}

void MemoryPathController::removePath ( std::string path )
{
    removeFromPathList(pathList, path);
    updatePaths();

}
