/**
 * @file	key-board-widget.h
 * @brief	Declares the key board widget class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Key-Board
 * Copyright (C)  2010 <>
 *
 * Key-Board is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Key-Board is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _KEY_BOARD_WIDGET_H_
#define _KEY_BOARD_WIDGET_H_

#include <gtkmm.h>
#include <string>
#include "key-board-widget-interface.h"
#include "smartmatic-button.h"
#include <Widgets/SmartmaticBasicTable.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @enum	KeysKeyboard
             * @brief	Values that represent KeysKeyboard.
             */
            enum KeysKeyboard
            {
                kbZero, kbOne, kbTwo, kbThree, kbFour, kbFive, kbSix, kbSeven,  ///< Values that represent Keys Zero, One, Two, Three, Four, Five, Six and Seven.
                kbEight, kbNine,	///< Values that represent Keys Eight and Nine.

                kbA, kbB, kbC, kbD, kbE, kbF, kbG, kbH, kbI, kbJ, kbK, kbL, kbM,	///< Values that represent Keys A, B, C, D, E, F, G, H, I, J, K, L and M.
                kbN, kbO, kbP, kbQ, kbR, kbS, kbT, kbU, kbV, kbW, kbX, kbY, kbZ,	///< Values that represent Keys N, O, P, Q, R, S, T, U, V, W, X, Y and Z.

                kbBackspace, kbEnter, kbEsc, kbSpace, kbComa, kbPoint,  ///< Values that represent Keys Backspace, Enter, Esc, Space, Coma and Point.
                kbDest, kbUnder, kbEqual, kbBrace1, kbBrace2, kbBackslash, kbApostrophe,	///< Values that represent Keys Dest, Under, Equal, Brace1, Brace2, Backslash and Apostrophe.
                kbShift, kbNN, kbSlash, kbUp, kbDown, kbLeft, kbRight   ///< Values that represent Keys Shift, NN, Slash, Up, Down, Left and Right.
            };


            /**
             * @class	Keyboard
             * @brief	Keyboard.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class Keyboard: public SmartmaticBasicTable
            {
                public:
                    //Constructor

                    /**
                     * @fn	Keyboard::Keyboard();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param   capsOffByDefault Whether the capital letters are off by default.
                     */
                    explicit Keyboard(const bool capsOffByDefault = false);

                    //Destructor

                    /**
                     * @fn	virtual Keyboard::~Keyboard();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~Keyboard();

                    //Signal Handlers
                    sigc::signal<void, std::string> signal_detected; ///< The signal detected
                    sigc::signal<void> signalDelete_detected;   ///< The signal delete detected
                    sigc::signal<void> signalEnter_detected;	///< The signal enter detected
                    sigc::signal<void> signalUp_detected;	///< The signal up detected
                    sigc::signal<void> signalDown_detected;	///< The signal down detected
                    sigc::signal<void> signalLeft_detected;	///< The signal left detected
                    sigc::signal<void> signalRight_detected;	///< The signal right detected

                    /**
                     * @fn	void Keyboard::buttonArraySetNames(std::string prefixName);
                     * @brief	Button array set names.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	prefixName	Name of the prefix.
                     */
                    void buttonArraySetNames ( std::string prefixName );

                    /**
                     * @fn	void setSensitiveButton ( Smartmatic::GUI::Widgets::KeysKeyboard key, bool sensitive = false );
                     * @brief	set sensitive to button
                     * @author	Yadiockson Soto
                     * @date	06/09/2013
                     * @param	key	The key value from button
                     * @param sensitive new value
                     */
                    void setSensitiveButton ( Smartmatic::GUI::Widgets::KeysKeyboard key, bool sensitive = false );

                protected:
                    //Signal Handlers

                    /**
                     * @fn	void Keyboard::on_button_keynumbered(std::string data);
                     * @brief	Handles button keynumbered signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	data	The data.
                     */

                    void on_button_keynumbered ( std::string data );

                private:

                    SmartmaticBasicTable * keyBoardTable; ///< The key board table
                    SmartmaticButton * buttonArray[56]; ///< Array of buttons

                    /**
                     * @fn	void Keyboard::on_button_shift();
                     * @brief	Handles button shift signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    void on_button_shift();
                    bool isShifted; ///< true if is shifted

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };
        }
    }
}

#endif // _KEY_BOARD_WIDGET_H_
