#include "TestsManager.hxx"

#include <fstream>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <string>
#include <unistd.h>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOM.hpp>

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <System/IO/Directory.h>
#include <Voting/Election/election-schema.hxx>
#include <Voting/VotingDevice/logical-voter-group-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/Vote/VoteFactory.hxx>

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::IO;
using namespace xercesc;

static ISMTTLog* m_logger = SMTTLogManager::GetLogger("conformity-tests-tool", "conformity-tests-tool");

#ifdef CONFORMITY_TESTS_TOOL_CONFIG_DIR
static const std::string CONFORMITY_TESTS_CONFIG_PATH                          = CONFORMITY_TESTS_TOOL_CONFIG_DIR;
#else
static const std::string CONFORMITY_TESTS_CONFIG_PATH                          = ".";
#endif

static const std::string VOTE_ELEMENT_NAME                                     = "vote";
static const std::string VOTE_CODE_ATTRIBUTE_NAME                              = "code";
static const std::string VOTE_LOGICAL_VOTER_GROUP_CODE_ATTRIBUTE_NAME          = "logical-voter-group-code";

static const std::string SELECTIONS_ELEMENT_NAME                               = "selections";
static const std::string SELECTION_ELEMENT_NAME                                = "selection";

static const std::string CONTEST_ELEMENT_NAME                                  = "contest";
static const std::string CONTEST_CODE_ATTRIBUTE_NAME                           = "code";
static const std::string SELECTION_CONTEST_CODE_ATTRIBUTE_NAME                 = "contest-code";

static const std::string ELIGIBLE_OPTIONS_ELEMENT_NAME                         = "eligible-options";
static const std::string ELIGIBLE_OPTION_ELEMENT_NAME                          = "eligible-option";
static const std::string ELIGIBLE_OPTION_CODE_ATTRIBUTE_NAME                   = "code";
static const std::string ELIGIBLE_OPTION_PARENT_CODE_ATTRIBUTE_NAME            = "parent-code";

static const std::string BALLOT_OPTION_ELEMENT_NAME                            = "ballot-option";
static const std::string BALLOT_OPTIONS_ELEMENT_NAME                           = "ballot-options";
static const std::string BALLOT_OPTION_CODE_ATTRIBUTE_NAME                     = "code";
static const std::string PARTY_NAME_TO_PRINT_ATTRIBUTE_NAME                    = "party-name-to-print";
static const std::string BALLOT_OPTION_CATEGORY_ATTRIBUTE_NAME                 = "category";

static const std::string PARTY_ELEMENT_NAME                                    = "party";
static const std::string PARTY_CODE_ATTRIBUTE_NAME                             = "code";
static const std::string PARTY_NAME_ATTRIBUTE_NAME                             = "name";

static const std::string LOGICAL_VOTER_GROUP_ELEMENT_NAME                      = "logical-voter-group";
static const std::string LOGICAL_VOTER_GROUP_CODE_ATTRIBUTE_NAME               = "code";
static const std::string LOGICAL_VOTER_GROUP_CONTEST_GROUP_CODE_ATTRIBUTE_NAME = "contest-group-code";

static const std::string CONTEST_GROUP_ELEMENT_NAME                            = "contest-group";
static const std::string CONTEST_GROUP_CODE_ATTRIBUTE_NAME                     = "code";
static const std::string CONTEST_CODE_ELEMENT_NAME                             = "contest-code";

static const std::string TOTAL_VOTES_ATTRIBUTE_NAME                            = "total-votes";

static const std::string CONTEST_RESULTS_ELEMENT_NAME                          = "contest-results";
static const std::string CONTEST_RESULTS_CODE_ATTRIBUTE_NAME                   = "code";

static const std::string PARTY_RESULTS_ELEMENT_NAME                            = "party-results";
static const std::string PARTY_RESULTS_CODE_ATTRIBUTE_NAME                     = "code";
static const std::string PARTY_LIST_VOTES_ATTRIBUTE_NAME                       = "list-votes";
static const std::string PARTY_TOTAL_VOTES_ATTRIBUTE_NAME                      = "total-votes";

static const std::string CANDIDATE_RESULTS_ELEMENT_NAME                        = "candidate-results";
static const std::string CANDIDATE_ELEMENT_NAME                                = "candidate";
static const std::string CANDIDATE_CODE_ATTRIBUTE_NAME                         = "code";
static const std::string CANDIDATE_PREFERENCE_VOTES_ATTRIBUTE_NAME             = "preference-votes";

static const std::string SUBSTITUTE_RESULTS_ELEMENT_NAME                       = "substitute-results";
static const std::string SUBSTITUTE_ELEMENT_NAME                               = "substitute";
static const std::string SUBSTITUTE_CODE_ATTRIBUTE_NAME                        = "code";
static const std::string SUBSTITUTE_PREFERENCE_VOTES_ATTRIBUTE_NAME            = "preference-votes";

static const std::string CONFIGURATION_FILE_NAME                               = "SETTINGS";

static const std::string ELECTION_FILE_NAME_SETTING                            = "election-file-name";
static const std::string DEFAULT_ELECTION_FILE_NAME                            = "election.xml";
static const std::string VOTES_FILE_NAME_SETTING                               = "votes-file-name";
static const std::string DEFAULT_VOTES_FILE_NAME                               = "votes.xml";
static const std::string VOTE_RESULTS_FILE_NAME_SETTING                        = "vote-results-file-name";
static const std::string DEFAULT_VOTE_RESULTS_FILE_NAME                        = "vote-results.xml";

static const std::string CHECK_MINIMUM_NUMBER_OF_SELECTIONS_SETTING            = "check-minimum-number-of-selections";
static const std::string CHECK_SELECTIONS_BELONG_TO_SAME_PARTY_SETTING         = "check-selections-belong-to-same-party";
static const std::string CHECK_SELECTIONS_APPLICABLE_TO_CONSTITUENCY_SETTING   = "check-selections-applicable-to-constituency";
static const std::string CHECK_UNIQUENESS_IN_SELECTIONS_SETTING                = "check-uniqueness-in-selections";
static const std::string CHECK_LIST_HEADER_RESOURCE_CORRECTNESS_SETTING        = "check-list-header-resource-correctness";
static const std::string CHECK_CUMULATIVE_NUMBER_OF_SELECTIONS_SETTING         = "check-cumulative-number-of-selections";

static const std::string CHECK_MINIMUM_NUMBER_OF_SCANNINGS_SETTING             = "check-minimum-number-of-scannings";
static const std::string CHECK_SCANNINGS_BELONG_TO_SAME_PARTY_SETTING          = "check-scannings-belong-to-same-party";
static const std::string CHECK_SCANNINGS_APPLICABLE_TO_CONSTITUENCY_SETTING    = "check-scannings-applicable-to-constituency";
static const std::string CHECK_UNIQUENESS_IN_SCANNINGS_SETTING                 = "check-uniqueness-in-scannings";
static const std::string CHECK_CUMULATIVE_NUMBER_OF_SCANNINGS_SETTING          = "check-cumulative-number-of-scannings";

static const std::string CHECK_TOTAL_CUMULATIVE_SELECTIONS_SETTING             = "check-total-cumulative-selections";
static const std::string CHECK_INDIVIDUAL_CUMULATIVE_SELECTIONS_SETTING        = "check-individual-cumulative-selections";

static Eligible_option *buildEligibleOption(
    const std::string &eligibleOptionCode,
    const std::string &eligibleOptionParentCode,
    const std::string &ballotOptionCode,
    const std::string &ballotOptionCategory)
{
    Ballot_option ballotOption;
    ballotOption.setCode(ballotOptionCode);

    if (ballotOptionCategory == "PRINCIPAL")
    {
        ballotOption.setCategory(Category::ELECTION_PRINCIPAL);
    }
    else
    {
        ballotOption.setCategory(Category::ELECTION_STAND_IN);
    }

    std::auto_ptr < Ballot_options > ballotOptionsList;
    ballotOptionsList.reset(new Ballot_options());
    Ballot_options::Ballot_optionSequence ballotOptionsSequence;
    ballotOptionsSequence.push_back(ballotOption);
    ballotOptionsList->setBallot_option(ballotOptionsSequence);

    Eligible_option *eligibleOption = new Eligible_option();
    eligibleOption->setCode(eligibleOptionCode);
    eligibleOption->setParent_code(eligibleOptionParentCode);
    eligibleOption->setBallot_options(ballotOptionsList);

    return eligibleOption;
}

static ItemResult buildItemResult(
    const std::string &ballotOptionCode,
    const size_t preferenceVotes)
{
    ItemResult candidateResult;
    candidateResult.setCode(ballotOptionCode);
    candidateResult.setPreference_votes(preferenceVotes);

    return candidateResult;
}

static Logical_voter_group *buildLogicalVoterGroup(const std::string &code, const std::string &contestGroupCode)
{
    Logical_voter_group *logicalVoterGroup = new Logical_voter_group();
    logicalVoterGroup->setCode(code);
    logicalVoterGroup->setContest_group_code(contestGroupCode);

    return logicalVoterGroup;
}

static Contest_group *buildContestGroup(
    const std::string &contestGroupCode,
    const std::list<std::string> contestCodesList)
{
    Contest_group::Contest_codeSequence contestsSequence;

    std::list<std::string>::const_iterator it = contestCodesList.begin();
    const std::list<std::string>::const_iterator end = contestCodesList.end();

    for (; it != end; ++it)
    {
        contestsSequence.push_back(*it);
    }

    Contest_group *contestGroup = new Contest_group();
    contestGroup->setCode(contestGroupCode);
    contestGroup->setContest_code(contestsSequence);

    return contestGroup;
}

TestsManager::TestsManager()
: m_electionFilePath("")
, m_votesFilePath("")
, m_voteResultsFilePath("")
, m_electionFileFound(false)
, m_votesFileFound(false)
, m_voteResultsFileFound(false)
, m_vote(NULL)
{
    XMLPlatformUtils::Initialize();

    m_electionXml    = new XercesDOMParser();
    m_votesXml       = new XercesDOMParser();
    m_voteResultsXml = new XercesDOMParser();

    m_votesList.clear();
    m_partyTextToPrintMapList.clear();
    m_logicalVoterGroupsList.clear();
    m_contestGroupByCodeMap.clear();

    initialize();
}

TestsManager::~TestsManager()
{
    delete m_electionXml;
    delete m_votesXml;
    delete m_voteResultsXml;
    delete m_vote;
}

void TestsManager::initialize()
{
    parseSettingsFile();

    m_electionFileFound    = Directory::getInstance()->FileExists(m_electionFilePath);
    m_votesFileFound       = Directory::getInstance()->FileExists(m_votesFilePath);
    m_voteResultsFileFound = Directory::getInstance()->FileExists(m_voteResultsFilePath);

    if (m_electionFileFound)
    {
        if ((m_votesFileFound == false) && (m_voteResultsFileFound == false))
        {
            m_logger->Error("You must include at least the votes file or the votes results file in '" +
                CONFORMITY_TESTS_CONFIG_PATH + "'.");
        }
        else
        {
            parseElectionFile();

            if (m_votesFileFound)
            {
                parseVotesFile();
            }

            if (m_voteResultsFileFound)
            {
                parseVoteResultsFile();
            }
        }
    }
    else
    {
        m_logger->Error("You must include an electoral configuration file in '" + CONFORMITY_TESTS_CONFIG_PATH + "'.");
    }
}

void TestsManager::parseSettingsFile()
{
    m_configurationManager.initialize();
    m_electionFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
        DEFAULT_ELECTION_FILE_NAME : CONFORMITY_TESTS_CONFIG_PATH + "/" + DEFAULT_ELECTION_FILE_NAME;
    m_votesFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
        DEFAULT_VOTES_FILE_NAME : CONFORMITY_TESTS_CONFIG_PATH + "/" + DEFAULT_VOTES_FILE_NAME;
    m_voteResultsFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
        DEFAULT_VOTE_RESULTS_FILE_NAME : CONFORMITY_TESTS_CONFIG_PATH + "/" + DEFAULT_VOTE_RESULTS_FILE_NAME;

    const std::string configurationFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
        CONFIGURATION_FILE_NAME : CONFORMITY_TESTS_CONFIG_PATH + "/" + CONFIGURATION_FILE_NAME;

    if (Directory::getInstance()->FileExists(configurationFilePath) == false)
    {
        m_logger->Debug("Settings file 'SETTINGS' was not found in '" + CONFORMITY_TESTS_CONFIG_PATH +
            "'. Default configurations will be used.");
        return;
    }

    std::ifstream inputFileStream(configurationFilePath.c_str());
    std::string setting;

    while (std::getline(inputFileStream, setting, '='))
    {
        std::string stringValue;

        if (std::getline(inputFileStream, stringValue))
        {
            if (setting == ELECTION_FILE_NAME_SETTING)
            {
                m_electionFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
                    stringValue : CONFORMITY_TESTS_CONFIG_PATH + "/" + stringValue;
            }

            if (setting == VOTES_FILE_NAME_SETTING)
            {
                m_votesFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
                    stringValue : CONFORMITY_TESTS_CONFIG_PATH + "/" + stringValue;
            }

            if (setting == VOTE_RESULTS_FILE_NAME_SETTING)
            {
                m_voteResultsFilePath = (CONFORMITY_TESTS_CONFIG_PATH == ".") ?
                    stringValue : CONFORMITY_TESTS_CONFIG_PATH + "/" + stringValue;
            }

            if (setting == CHECK_MINIMUM_NUMBER_OF_SELECTIONS_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckMinimumNumberOfSelections((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_SELECTIONS_BELONG_TO_SAME_PARTY_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckSelectionsBelongToSameParty((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_SELECTIONS_APPLICABLE_TO_CONSTITUENCY_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckSelectionsApplicableToConstituency((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_UNIQUENESS_IN_SELECTIONS_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckUniquenessInSelections((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_LIST_HEADER_RESOURCE_CORRECTNESS_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckListHeaderResourceCorrectness((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_CUMULATIVE_NUMBER_OF_SELECTIONS_SETTING)
            {
                m_configurationManager.getBeforePrintingVoteComplianceVerificationConfiguration()
                    .setCheckCumulativeNumberOfSelections((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_MINIMUM_NUMBER_OF_SCANNINGS_SETTING)
            {
                m_configurationManager.getAfterScanningVoteComplianceVerificationConfiguration()
                    .setCheckMinimumNumberOfScannings((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_SCANNINGS_BELONG_TO_SAME_PARTY_SETTING)
            {
                m_configurationManager.getAfterScanningVoteComplianceVerificationConfiguration()
                    .setCheckScanningsBelongToSameParty((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_SCANNINGS_APPLICABLE_TO_CONSTITUENCY_SETTING)
            {
                m_configurationManager.getAfterScanningVoteComplianceVerificationConfiguration()
                    .setCheckScanningsApplicableToConstituency((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_UNIQUENESS_IN_SCANNINGS_SETTING)
            {
                m_configurationManager.getAfterScanningVoteComplianceVerificationConfiguration()
                    .setCheckUniquenessInScannings((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_CUMULATIVE_NUMBER_OF_SCANNINGS_SETTING)
            {
                m_configurationManager.getAfterScanningVoteComplianceVerificationConfiguration()
                    .setCheckCumulativeNumberOfScannings((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_TOTAL_CUMULATIVE_SELECTIONS_SETTING)
            {
                m_configurationManager.getBeforeGeneratingResultsComplianceVerificationConfiguration()
                    .setCheckTotalCumulativeSelections((stringValue == "TRUE") ? true : false);
            }

            if (setting == CHECK_INDIVIDUAL_CUMULATIVE_SELECTIONS_SETTING)
            {
                m_configurationManager.getBeforeGeneratingResultsComplianceVerificationConfiguration()
                    .setCheckIndividualCumulativeSelections((stringValue == "TRUE") ? true : false);
            }
        }
    }

    SAESFunctionalityConfigurationManager::getInstance(&m_configurationManager);
}

void TestsManager::parseElectionFile()
{
    try
    {
        m_electionXml->parse(m_electionFilePath.c_str());
    }
    catch (const XMLException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (const DOMException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (...)
    {
        m_logger->Error("Unexpected exception");
    }
}

void TestsManager::parseVotesFile()
{
    try
    {
        m_votesXml->parse(m_votesFilePath.c_str());
    }
    catch (const XMLException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (const DOMException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (...)
    {
        m_logger->Error("Unexpected exception");
    }
}

void TestsManager::parseVoteResultsFile()
{
    try
    {
        m_voteResultsXml->parse(m_voteResultsFilePath.c_str());
    }
    catch (const XMLException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (const DOMException& exception)
    {
        std::string log = XMLString::transcode(exception.getMessage());
        m_logger->Error("Exception in XML: " + log);
    }
    catch (...)
    {
        m_logger->Error("Unexpected exception");
    }
}

void TestsManager::checkCurrentContest(const DOMNode * node)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute =
            attributes->getNamedItem(XMLString::transcode(CONTEST_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code            = XMLString::transcode(codeAttribute->getNodeValue());

        m_partyListMap.clear();
        m_partyPrincipalMap.clear();
        m_partyStandInMap.clear();

        iterateOverElectionFileContests(node, code);
    }
}

void TestsManager::checkCurrentVote(const DOMNode * node)
{
    m_vote = VoteFactory::createVote();
    m_vote->getSelections().clear();
    m_selectionsSequence.clear();
    m_partyTextToPrintMap.clear();

    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute =
            attributes->getNamedItem(XMLString::transcode(VOTE_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code            = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * logicalVoterGroupCodeAttribute =
            attributes->getNamedItem(XMLString::transcode(VOTE_LOGICAL_VOTER_GROUP_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string logicalVoterGroupCode            =
            XMLString::transcode(logicalVoterGroupCodeAttribute->getNodeValue());

        m_vote->setCode(code);
        m_vote->setLogicalVoterGroup(logicalVoterGroupCode);

        iterateOverVotesFile(node);

        if (m_vote->getSelections().size() > 0)
        {
            m_votesList.push_back(m_vote);
            m_partyTextToPrintMapList.push_back(m_partyTextToPrintMap);
        }
    }
}

void TestsManager::checkCurrentSelection(const DOMNode * node)
{
    m_ballotOptionsSequence.clear();

    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * contestCodeAttribute =
            attributes->getNamedItem(XMLString::transcode(SELECTION_CONTEST_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string contestCode            = XMLString::transcode(contestCodeAttribute->getNodeValue());

        m_selection.setContest(contestCode);

        if (node->getChildNodes()->getLength() == 0)
        {
            m_selection.getBallotOptions().clear();
        }

        iterateOverVotesFile(node, contestCode);

        m_selectionsSequence.push_back(m_selection);
        m_vote->setSelections(m_selectionsSequence);
    }
}

void TestsManager::checkCurrentEligibleOption(const DOMNode * node, const std::string &contestCode)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute       =
            attributes->getNamedItem(XMLString::transcode(ELIGIBLE_OPTION_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code                  = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * parentCodeAttribute =
            attributes->getNamedItem(XMLString::transcode(ELIGIBLE_OPTION_PARENT_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string parentCode            = XMLString::transcode(parentCodeAttribute->getNodeValue());

        iterateOverElectionFileContests(node, contestCode, code, parentCode);
    }
}

void TestsManager::checkCurrentBallotOption(
    const DOMNode * node,
    const std::string &contestCode,
    const std::string &eligibleOptionCode,
    const std::string &eligibleOptionParentCode)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute     =
            attributes->getNamedItem(XMLString::transcode(BALLOT_OPTION_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code                =
            XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * categoryAttribute =
            attributes->getNamedItem(XMLString::transcode(BALLOT_OPTION_CATEGORY_ATTRIBUTE_NAME.c_str()));
        const std::string category            = XMLString::transcode(categoryAttribute->getNodeValue());

        iterateOverElectionFileContests(node, contestCode, eligibleOptionCode, eligibleOptionParentCode, code, category);
    }
}

void TestsManager::checkSelectionBallotOption(const DOMNode * node, const std::string &contestCode)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute             =
            attributes->getNamedItem(XMLString::transcode(BALLOT_OPTION_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code                        = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * partyNameToPrintAttribute =
            attributes->getNamedItem(XMLString::transcode(PARTY_NAME_TO_PRINT_ATTRIBUTE_NAME.c_str()));
        const std::string partyNameToPrint            = XMLString::transcode(partyNameToPrintAttribute->getNodeValue());

        m_partyTextToPrintMap[contestCode][code] = partyNameToPrint;
        m_ballotOptionsSequence.push_back(code);
        m_selection.setBallotOptions(m_ballotOptionsSequence);
    }
}

void TestsManager::checkCurrentParty(
    const DOMNode * node,
    const std::string &contestCode,
    const std::string &eligibleOptionCode,
    const std::string &eligibleOptionParentCode,
    const std::string &ballotOptionCode,
    const std::string &ballotOptionCategory)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute =
            attributes->getNamedItem(XMLString::transcode(PARTY_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string   code          =
            XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * nameAttribute =
            attributes->getNamedItem(XMLString::transcode(PARTY_NAME_ATTRIBUTE_NAME.c_str()));
        const std::string   name          =
            XMLString::transcode(nameAttribute->getNodeValue());

        Eligible_option *eligibleOption =
            buildEligibleOption(eligibleOptionCode, eligibleOptionParentCode, ballotOptionCode, ballotOptionCategory);

        setEligibleOptionsByContestParty(eligibleOption, contestCode, code);
        setEligibleOptionsByBallotOptions(eligibleOption, contestCode);
        setPartyListByContest(contestCode, code, name);
        setPartyCodeByBallotOptionCode(ballotOptionCode, code);
    }
}

void TestsManager::checkVoteResults(const DOMNode * node)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * totalVotesAttribute =
            attributes->getNamedItem(XMLString::transcode(TOTAL_VOTES_ATTRIBUTE_NAME.c_str()));
        std::istringstream totalVotesValue(XMLString::transcode(totalVotesAttribute->getNodeValue()));
        totalVotesValue >> m_totalVotes;

        m_contestsSequence.clear();

        iterateOverVoteResultsFile(node, m_totalVotes);

        m_voteResults.setContestResults(m_contestsSequence);
    }
}

void TestsManager::checkContestResults(const DOMNode * node, const size_t totalVotes)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute =
            attributes->getNamedItem(XMLString::transcode(CONTEST_RESULTS_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string   code          =
            XMLString::transcode(codeAttribute->getNodeValue());

        m_partiesSequence.clear();

        iterateOverVoteResultsFile(node, totalVotes, code);

        m_contestResults.setCode(code);
        m_contestResults.setPartyResults(m_partiesSequence);
        m_contestsSequence.push_back(m_contestResults);
    }
}

void TestsManager::checkPartyResults(const DOMNode * node, const size_t totalVotes, const std::string &contestCode)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute            =
            attributes->getNamedItem(XMLString::transcode(PARTY_RESULTS_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string   code                     = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * partyListVotesAttribute =
            attributes->getNamedItem(XMLString::transcode(PARTY_LIST_VOTES_ATTRIBUTE_NAME.c_str()));
        size_t partyListVotes;
        std::istringstream listVotesValue(XMLString::transcode(partyListVotesAttribute->getNodeValue()));
        listVotesValue >> partyListVotes;

        const DOMNode     * partyTotalVotesAttribute =
            attributes->getNamedItem(XMLString::transcode(PARTY_TOTAL_VOTES_ATTRIBUTE_NAME.c_str()));
        size_t partyTotalVotes;
        std::istringstream totalVotesValue(XMLString::transcode(partyTotalVotesAttribute->getNodeValue()));
        totalVotesValue >> partyTotalVotes;

        m_candidatesSequence.clear();
        m_substitutesSequence.clear();

        iterateOverVoteResultsFile(node, totalVotes, contestCode);

        m_candidatesResults.setCandidate(m_candidatesSequence);
        m_substitutesResults.setCandidate(m_substitutesSequence);

        m_partyResults.setCode(code);
        m_partyResults.setCandidateResults(m_candidatesResults);
        m_partyResults.setSubstituteResults(m_substitutesResults);
        m_partyResults.setList_votes(partyListVotes);
        m_partyResults.setTotal_votes(partyTotalVotes);

        m_partiesSequence.push_back(m_partyResults);
    }
}

void TestsManager::checkCandidateResult(const DOMNode * node)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute                     =
            attributes->getNamedItem(XMLString::transcode(CANDIDATE_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string   code                              = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * candidatePreferenceVotesAttribute =
            attributes->getNamedItem(XMLString::transcode(CANDIDATE_PREFERENCE_VOTES_ATTRIBUTE_NAME.c_str()));
        size_t candidatePreferenceVotes;
        std::istringstream preferenceVotesValue(XMLString::transcode(candidatePreferenceVotesAttribute->getNodeValue()));
        preferenceVotesValue >> candidatePreferenceVotes;

        m_candidatesSequence.push_back(buildItemResult(code, candidatePreferenceVotes));
    }
}

void TestsManager::checkSubstituteResult(const DOMNode * node)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute                     =
            attributes->getNamedItem(XMLString::transcode(SUBSTITUTE_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string   code                              = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * substitutePreferenceVotesAttribute =
            attributes->getNamedItem(XMLString::transcode(SUBSTITUTE_PREFERENCE_VOTES_ATTRIBUTE_NAME.c_str()));
        size_t substitutePreferenceVotes;
        std::istringstream preferenceVotesValue(XMLString::transcode(substitutePreferenceVotesAttribute->getNodeValue()));
        preferenceVotesValue >> substitutePreferenceVotes;

        m_substitutesSequence.push_back(buildItemResult(code, substitutePreferenceVotes));
    }
}

void TestsManager::setEligibleOptionsByContestParty(
    Eligible_option *eligibleOption,
    const std::string &contestCode,
    const std::string &partyCode)
{
    if (eligibleOption->getParent_code().empty())
    {
        m_partyListMap[partyCode].push_back(eligibleOption);
        m_mapEligibleOptionsListByContestParty[contestCode] = m_partyListMap;
    }
    else if (eligibleOption->getBallot_options().getBallot_option()[0].getCategory() == Category::ELECTION_PRINCIPAL)
    {
        m_partyPrincipalMap[partyCode].push_back(eligibleOption);
        m_mapEligibleOptionsPrincipalByContestParty[contestCode] = m_partyPrincipalMap;
    }
    else
    {
        m_partyStandInMap[partyCode].push_back(eligibleOption);
        m_mapEligibleOptionsStandInByContestParty[contestCode] = m_partyStandInMap;
    }
}

void TestsManager::setEligibleOptionsByBallotOptions(
    Eligible_option *eligibleOption,
    const std::string &contestCode)
{
    const std::string ballotOptionCode = eligibleOption->getBallot_options().getBallot_option()[0].getCode();
    m_eligibleOptionByBallotOptionCodeMap[contestCode][ballotOptionCode] = eligibleOption;
}

void TestsManager::setPartyListByContest(
    const std::string &contestCode,
    const std::string &partyCode,
    const std::string &partyName)
{
    bool found = false;
    std::list< Party* >::const_iterator it = m_contestPartyMap[contestCode].begin();
    const std::list < Party* >::const_iterator end = m_contestPartyMap[contestCode].end();

    for (; it != end; ++it)
    {
        if ((*it)->getCode() == partyCode)
        {
            found = true;
            break;
        }
    }

    if (found == false)
    {
        Party *party = new Party();
        party->setCode(partyCode);
        party->setName(partyName);
        m_contestPartyMap[contestCode].push_back(party);
    }
}

void TestsManager::setPartyCodeByBallotOptionCode(const std::string &ballotOptionCode, const std::string &partyCode)
{
    m_partyCodeByBallotOptionCodeMap[ballotOptionCode] = partyCode;
}

void TestsManager::iterateOverElectionFileContests(
    const DOMNode * node,
    const std::string &contestCode,
    const std::string &eligibleOptionCode,
    const std::string &eligibleOptionParentCode,
    const std::string &ballotOptionCode,
    const std::string &ballotOptionCategory)
{
    const DOMNodeList * childNodes = node->getChildNodes();

    for (size_t i = 0; i < childNodes->getLength(); ++i)
    {
        if (childNodes->item(i)->getNodeType() == DOMNode::ELEMENT_NODE)
        {
            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CONTEST_ELEMENT_NAME)
            {
                checkCurrentContest(childNodes->item(i));
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == ELIGIBLE_OPTIONS_ELEMENT_NAME)
            {
                iterateOverElectionFileContests(childNodes->item(i), contestCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == ELIGIBLE_OPTION_ELEMENT_NAME)
            {
                checkCurrentEligibleOption(childNodes->item(i), contestCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == BALLOT_OPTIONS_ELEMENT_NAME)
            {
                iterateOverElectionFileContests(childNodes->item(i), contestCode, eligibleOptionCode, eligibleOptionParentCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == BALLOT_OPTION_ELEMENT_NAME)
            {
                checkCurrentBallotOption(childNodes->item(i), contestCode, eligibleOptionCode, eligibleOptionParentCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == PARTY_ELEMENT_NAME)
            {
                checkCurrentParty(
                    childNodes->item(i),
                    contestCode,
                    eligibleOptionCode,
                    eligibleOptionParentCode,
                    ballotOptionCode,
                    ballotOptionCategory);
            }
        }
    }
}

void TestsManager::checkCurrentLogicalVoterGroup(const xercesc::DOMNode * node)
{
    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute             =
            attributes->getNamedItem(XMLString::transcode(LOGICAL_VOTER_GROUP_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code                        = XMLString::transcode(codeAttribute->getNodeValue());

        const DOMNode     * contestGroupCodeAttribute =
            attributes->getNamedItem(XMLString::transcode(LOGICAL_VOTER_GROUP_CONTEST_GROUP_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string contestGroupCode            = XMLString::transcode(contestGroupCodeAttribute->getNodeValue());

        Logical_voter_group *logicalVoterGroup = buildLogicalVoterGroup(code, contestGroupCode);

        m_logicalVoterGroupsList.push_back(logicalVoterGroup);
    }
}

void TestsManager::iterateOverElectionFileLogicalVoterGroups(
    const xercesc::DOMNode * node,
    const std::string &logicalVoterGroupCode,
    const std::string &logicalVoterGroupContestGroupCode)
{
    const DOMNodeList * childNodes = node->getChildNodes();

    for (size_t i = 0; i < childNodes->getLength(); ++i)
    {
        if (XMLString::transcode(childNodes->item(i)->getNodeName()) == LOGICAL_VOTER_GROUP_ELEMENT_NAME)
        {
            checkCurrentLogicalVoterGroup(childNodes->item(i));
        }
    }
}

void TestsManager::checkCurrentContestGroup(const xercesc::DOMNode * node)
{
    m_contestCodesList.clear();

    const DOMNamedNodeMap * attributes = node->getAttributes();

    if (attributes)
    {
        const DOMNode     * codeAttribute             =
            attributes->getNamedItem(XMLString::transcode(CONTEST_GROUP_CODE_ATTRIBUTE_NAME.c_str()));
        const std::string code                        = XMLString::transcode(codeAttribute->getNodeValue());

        iterateOverElectionFileContestGroups(node, code);

        m_contestGroupByCodeMap[code] = buildContestGroup(code, m_contestCodesList);
    }
}

void TestsManager::checkCurrentContestCode(
    const xercesc::DOMNode * node,
    const std::string &contestGroupCode)
{
    m_contestCodesList.push_back(XMLString::transcode(node->getTextContent()));
}

void TestsManager::iterateOverElectionFileContestGroups(
    const xercesc::DOMNode * node,
    const std::string &contestGroupCode,
    const std::string &contestCode)
{
    const DOMNodeList * childNodes = node->getChildNodes();

    for (size_t i = 0; i < childNodes->getLength(); ++i)
    {
        if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CONTEST_GROUP_ELEMENT_NAME)
        {
            checkCurrentContestGroup(childNodes->item(i));
        }

        if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CONTEST_CODE_ELEMENT_NAME)
        {
            checkCurrentContestCode(childNodes->item(i), contestGroupCode);
        }
    }
}

void TestsManager::iterateOverVotesFile(const DOMNode * node, const std::string &contestCode)
{
    const DOMNodeList * childNodes = node->getChildNodes();

    for (size_t i = 0; i < childNodes->getLength(); ++i)
    {
        if (childNodes->item(i)->getNodeType() == DOMNode::ELEMENT_NODE)
        {
            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == VOTE_ELEMENT_NAME)
            {
                checkCurrentVote(childNodes->item(i));
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == SELECTIONS_ELEMENT_NAME)
            {
                iterateOverVotesFile(childNodes->item(i));
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == SELECTION_ELEMENT_NAME)
            {
                checkCurrentSelection(childNodes->item(i));
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == BALLOT_OPTION_ELEMENT_NAME)
            {
                checkSelectionBallotOption(childNodes->item(i), contestCode);
            }
        }
    }
}

void TestsManager::iterateOverVoteResultsFile(
    const DOMNode * node,
    const size_t totalVotes,
    const std::string &contestCode)
{
    const DOMNodeList * childNodes = node->getChildNodes();

    for (size_t i = 0; i < childNodes->getLength(); ++i)
    {
        if (childNodes->item(i)->getNodeType() == DOMNode::ELEMENT_NODE)
        {
            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CONTEST_RESULTS_ELEMENT_NAME)
            {
                checkContestResults(childNodes->item(i), totalVotes);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == PARTY_RESULTS_ELEMENT_NAME)
            {
                checkPartyResults(childNodes->item(i), totalVotes, contestCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CANDIDATE_RESULTS_ELEMENT_NAME)
            {
                iterateOverVoteResultsFile(childNodes->item(i), totalVotes, contestCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == SUBSTITUTE_RESULTS_ELEMENT_NAME)
            {
                iterateOverVoteResultsFile(childNodes->item(i), totalVotes, contestCode);
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == CANDIDATE_ELEMENT_NAME)
            {
                checkCandidateResult(childNodes->item(i));
            }

            if (XMLString::transcode(childNodes->item(i)->getNodeName()) == SUBSTITUTE_ELEMENT_NAME)
            {
                checkSubstituteResult(childNodes->item(i));
            }
        }
    }
}

void TestsManager::initializeElectoralConfiguration()
{
    const DOMDocument * document = m_electionXml->getDocument();

    if (document == NULL)
    {
        m_logger->Error("Electoral configuration file empty, invalid or not found");
        return;
    }

    const DOMNode * electionNode = document->getFirstChild();
    const DOMNode * contestsNode = electionNode->getChildNodes()->item(1);
    const DOMNode * logicalVoterGroupsNode = electionNode->getChildNodes()->item(3);
    const DOMNode * contestGroupsNode = electionNode->getChildNodes()->item(5);

    iterateOverElectionFileContests(contestsNode);
    iterateOverElectionFileLogicalVoterGroups(logicalVoterGroupsNode);
    iterateOverElectionFileContestGroups(contestGroupsNode);

    m_mockElectoralConfiguration.setEligibleOptionsListByContestParty(m_mapEligibleOptionsListByContestParty);
    m_mockElectoralConfiguration.setEligibleOptionsPrincipalByContestParty(m_mapEligibleOptionsPrincipalByContestParty);
    m_mockElectoralConfiguration.setEligibleOptionsStandInByContestParty(m_mapEligibleOptionsStandInByContestParty);

    m_mockElectoralConfiguration.eoByBallotOptionCodeMap = m_eligibleOptionByBallotOptionCodeMap;
    m_mockElectoralConfiguration.setMapContestParty(m_contestPartyMap);
    m_mockElectoralConfiguration.setPartyCodeByBallotOptionCode(m_partyCodeByBallotOptionCodeMap);

    m_mockElectoralConfiguration.setLogicalVoterGroupsList(m_logicalVoterGroupsList);
    m_mockElectoralConfiguration.setContestGroupByCodeMap(m_contestGroupByCodeMap);
}

void TestsManager::initializeVotes()
{
    const DOMDocument * document = m_votesXml->getDocument();

    if (document == NULL)
    {
        m_logger->Error("Selections file empty, invalid or not found");
        return;
    }

    const DOMNode * votesNode = document->getFirstChild();

    iterateOverVotesFile(votesNode);
}

void TestsManager::initializeVoteResults()
{
    const DOMDocument * document = m_voteResultsXml->getDocument();

    if (document == NULL)
    {
        m_logger->Error("Vote results file empty, invalid or not found");
        return;
    }

    const DOMNode * voteResultsNode = document->getFirstChild();

    checkVoteResults(voteResultsNode);
}

bool TestsManager::getElectionFileFound()
{
    return m_electionFileFound;
}

bool TestsManager::getVotesFileFound()
{
    return m_votesFileFound;
}

bool TestsManager::getVoteResultsFileFound()
{
    return m_voteResultsFileFound;
}

void TestsManager::runConformityTests(const bool runIndividualVoteTests, const bool runVoteResultsTests)
{
    bool success = true;

    VoteComplianceVerificator * verificator = VoteComplianceVerificator::getInstance(&m_mockElectoralConfiguration);

    if ((runIndividualVoteTests) && (m_votesList.size() > 0))
    {
        std::list< Smartmatic::SAES::Voting::Vote::Vote* >::const_iterator votesIterator = m_votesList.begin();
        const std::list < Smartmatic::SAES::Voting::Vote::Vote* >::const_iterator votesEnd = m_votesList.end();

        std::list < std::map < std::string, std::map < std::string, std::string > > >::const_iterator textToPrintIterator =
            m_partyTextToPrintMapList.begin();
        const std::list < std::map < std::string, std::map < std::string, std::string > > >::const_iterator textToPrintEnd =
                m_partyTextToPrintMapList.end();

        m_logger->Debug("Starting to check individual vote tests.");

        for (; votesIterator != votesEnd; ++votesIterator)
        {
            success = verificator->checkAllBeforePrintingVoteConditions(**votesIterator, *textToPrintIterator);

            if (success == false)
            {
                m_logger->Error("An error has occurred while running before printing vote tests.");
                break;
            }

            success = verificator->checkAllAfterScanningVoteConditions(**votesIterator);

            if (success == false)
            {
                m_logger->Error("An error has occurred while running after scanning vote tests.");
                break;
            }

            ++textToPrintIterator;
        }
    }

    if (runIndividualVoteTests && success)
    {
        m_logger->Debug("Individual vote tests were executed successfully.");
    }

    if ((runVoteResultsTests) && (m_voteResults.getContestResults().size() > 0))
    {
        m_logger->Debug("Starting to check vote results tests.");
        success = verificator->checkAllBeforeGeneratingResultsConditions(m_totalVotes, m_voteResults);

        if (success)
        {
            m_logger->Debug("Vote results tests were executed successfully.");
        }
        else
        {
            m_logger->Error("An error has occurred while running vote results tests.");
        }
    }
}
