#!/bin/bash
#
# File: verifyDeliverable.sh
#
# Date: Wed Apr 20, 17:33:00 EST 2018.
# Author: marcel.mancini
#
# Revision: 3
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#
set -e

function processVerificationForEveryIndexFile() {
  local indexFilePath=$1
  local currentDir=$(pwd)
  local dirToEvaluate=$(dirname $1)
  local sha256IndexFile="sha256sum.txt"
  local signatureForIndexFile="sha256sum.sig"
  local temporalDiffFileToVerifyIndex="sha256.chk"
  cd $dirToEvaluate
  calculateSha256 $indexFilePath
  performDiff $sha256IndexFile $temporalDiffFileToVerifyIndex
  verifySignature $signatureForIndexFile $sha256IndexFile
  cd $currentDir
}

function calculateSha256() {
  printf "\nCalculating sha256 index for verification of directory tree contained in sha256 index file $1"
  find -path ./Documentation -prune -o -path ./ProofOfConcept -prune -o -type f -not -name sha256sum.txt -not -name sha256.chk -not -name  sha256sum.sig -not -name "*tar.gz" -not -name "*.tar" -not -name "*.zip" -exec sha256sum "{}" + | sort --key 2 > sha256.chk
}

function performDiff() {
  local fileToVerify=$1
  local temporalFileToCheckDiff=$2
  local currentDir=$(pwd)
  printf "\nVerifying if the sha256 index file has been changed\n"
  set +e
  local diff=$(diff $fileToVerify $temporalFileToCheckDiff)
  set -e
  rm -f  $temporalFileToCheckDiff
  if [ "$diff" != "" ]
  then
    printf $diff
    printf "\nError! In dir $currentDir the index file does not match for file $fileToVerify" 1>&2
    exit 64
  fi
}

function verifySignature() {
  local signatureToVerify=$1
  local fileForWhichSigWasGenerated=$2
  local currentDir=$(pwd)
  printf "\nVerifying signature of sha256 index file\n"
  set +e
  gpg --verify --trust-model always $signatureToVerify $fileForWhichSigWasGenerated
  local verify=$?
  set -e
  if [ "$verify" != 0 ]
  then
    printf "\nError! In dir $currentDir the signature could not be verified for file $signatureToVerify" 1>&2
    exit 125
  fi
}

function verifyExistenceOfSha256IndexInDir() {
  local -r sha256IndexFile="sha256sum.txt"
  if [ -z "$(ls -A $1)" ]; then
    printf "\nDirectory $1 is empty"
  else
    if [ -e $1/$sha256IndexFile ]; then
      printf "\nFile $sha256IndexFile existence verified for $1"
    else
      printf "\nError! The second level folder $1 does not contain an index file $sha256IndexFile, and it should" 1>&2
      exit 32
    fi
  fi
}

function verifyExistenceOfSha256IndexInEverySecondLevelDir() {
  printf "\nVeryfying existence of sha256 index files for every second level dir."
  find -maxdepth 2 -mindepth 2 -type d -print0 |  while IFS= read -rd $'\0' file; do
    if [[ ! ( ${file} =~ ^./Documentation || ${file} =~ ^./ProofOfConcept ) ]]; then
      verifyExistenceOfSha256IndexInDir $file
    fi
  done

}

function verifyExistenceOfSha256IndexPerEveryTarGzAndZip() {
  printf "\nVerifying existence of sha256 index file for every tar.gz and zip."
  find -path ./Documentation -prune -o -path ./ProofOfConcept -prune -o -type f -name '*.tar.gz' -print0 |  while IFS= read -rd $'\0' file; do
    processTarGzToFindRelatedSha256IndexFile $file
    createPackageChecksumAndVerify "$file"
  done
  find -path ./Documentation -prune -o -path ./ProofOfConcept -prune -o -type f -name '*.zip' -print0 |  while IFS= read -rd $'\0' file; do
    processZipToFindRelatedSha256IndexFile $file
    createPackageChecksumAndVerify "$file"
  done
  find -path ./Documentation -prune -o -path ./ProofOfConcept -prune -o -type f -name '*.tar' -print0 |  while IFS= read -rd $'\0' file; do
    processTarToFindRelatedSha256IndexFile $file
    createPackageChecksumAndVerify "$file"
  done
}

function processTarToFindRelatedSha256IndexFile(){
  local fullPath=$1
  local filenameWithoutExt="${fullPath%.tar}"
  local sha256IndexFile="$filenameWithoutExt-sha256sum.txt"
  if [ -e $sha256IndexFile ]; then
    printf "\nFile $sha256IndexFile existence verified for $fullPath"
  else
    printf "\nError! The file $fullPath does not have an associated index file $sha256IndexFile, and it should" 1>&2
    exit 32
  fi
}

processTarGzToFindRelatedSha256IndexFile(){
  local fullPath=$1
  local filenameWithoutExt="${fullPath%.tar.gz}"
  local sha256IndexFile="$filenameWithoutExt-sha256sum.txt"
  if [ -e $sha256IndexFile ]; then
    printf "\nFile $sha256IndexFile existence verified for $fullPath"
  else
    printf "\nError! The file $fullPath does not have an associated index file $sha256IndexFile, and it should" 1>&2
    exit 32
  fi
}

function processZipToFindRelatedSha256IndexFile(){
  local fullPath=$1
  local filenameWithoutExt="${fullPath%.zip}"
  local sha256IndexFile="$filenameWithoutExt-sha256sum.txt"
  if [ -e $sha256IndexFile ]; then
    printf "\nFile $sha256IndexFile existence verified for $fullPath"
  else
    printf "\nError! The file $fullPath does not have an associated index file $sha256IndexFile, and it should" 1>&2
    exit 32
  fi
}

function verifyEverySha256IndexFile() {
  find -type f -name "sha256sum.txt" -print0 | while IFS= read -rd $'\0' file; do
    printf "\nProcessing $file"
    processVerificationForEveryIndexFile "$file"
  done
}

function importPublicKeyOption() {
  printf "\nDo you want to import a public key (y/n)?"
  read -n 1 -p "Selection:" importInput
  if [ "$importInput" = "Y" ] || [ "$importInput" = "y" ]; then
    printf " Please input path of file with public key to import (full path to .asc or .gpg file)"
    read -p "Selection:" pgpFileInput
    set +e
    gpg --import $pgpFileInput
    set -e
    printf "These are the pgp keys you currently have in your default key ring"
    gpg --list-keys
  fi
}

function checkRootFoldersOfDeliverable() {
  declare -a arr=("Executables/ECM" "Executables/PM-VM" "Resources/ECM" "Resources/PM-VM" "SourceCode/ECM" "SourceCode/PM-VM")
  for i in "${arr[@]}"
  do
   local checkIfExists=$(find . -maxdepth 2 -mindepth 2 -path "*$i*") 
   if [ "$checkIfExists" = "" ] 
   then
    printf "\nError! The directory $i does not exist" 1>&2
    exit 16
  fi
  done
}

function askForCheckRootFoldersOfDeliverable() {
  printf "\nDo you want to check that the folders of Deliverable follow the proposed structure (y/n)?"
  read -n 1 -p "Selection:" checkFoldersInput
  if [ "$checkFoldersInput" = "Y" ] || [ "$checkFoldersInput" = "y" ]; then
    checkRootFoldersOfDeliverable
  fi
}

function createPackageChecksumAndVerify() {
  local -r package_name="$1"
  local -r tmp_dir="$(mktemp -d)"
  local -r package_file="$(readlink -e ${package_name})"

  cd "${tmp_dir}"

  if [[ "${package_name}" =~ \.tar\.gz$ ]]; then
    local -r checksum_file="${package_file%.tar.gz}-sha256sum.chk"
    local -r checksum_fileToVerify="${package_file%.tar.gz}-sha256sum.txt"
    local -r signature_file="${package_file%.tar.gz}-sha256sum.sig"
    $(tar xzf "${package_file}")
  elif [[ "${package_name}" =~ \.tar$ ]]; then
    local -r checksum_file="${package_file%.tar}-sha256sum.chk"
    local -r checksum_fileToVerify="${package_file%.tar}-sha256sum.txt"
    local -r signature_file="${package_file%.tar}-sha256sum.sig"
    $(tar xf "${package_file}")
  elif [[ "${package_name}" =~ \.zip$ ]]; then
    local -r checksum_file="${package_file%.zip}-sha256sum.chk"
    local -r checksum_fileToVerify="${package_file%.zip}-sha256sum.txt"
    local -r signature_file="${package_file%.zip}-sha256sum.sig"
    $(unzip "${package_file}" >/dev/null)
  fi

  $(find -path ./Documentation -prune -o -path ./PathOfConcept -prune -o -type f -print0 | xargs -0 sha256sum | sort -k 2 > ${checksum_file})
  printf "\nVerifying index file for $checksum_fileToVerify"
  performDiff $checksum_fileToVerify $checksum_file
  verifySignature $signature_file $checksum_fileToVerify

  cd - >/dev/null
  $(rm -rf ${tmp_dir})
}

function verifyThatProcessIsAppliedToDeliverablesFolder() {
  local verifyDeliverableFolder=$(basename $(pwd))
  if [ "$verifyDeliverableFolder" != "Deliverables" ]
  then
    printf "\nError! The path %path is not point to a Deliverables folder" 1>&2
    exit 2
  fi
}

function finish() {
  printf "\nCleaning up\n"
  cd $current_directory
}
trap finish EXIT

# Environment
readonly current_directory="$PWD"

function main() {
  local pathOfVerification=$1
  printf "\nVerification process will execute over $pathOfVerification"
  cd $pathOfVerification
  verifyThatProcessIsAppliedToDeliverablesFolder
  importPublicKeyOption
  askForCheckRootFoldersOfDeliverable
  verifyExistenceOfSha256IndexInEverySecondLevelDir
  verifyEverySha256IndexFile
  verifyExistenceOfSha256IndexPerEveryTarGzAndZip

  printf "\nVerification ended successfuly!"
}

main $@
exit 0

