/**
 * AsymmetricCipherPool.java
 *
 * $Revision: 1.4 $ ($Date: 2009-12-30 03:22:37 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.asymmetric;

import java.security.PrivateKey;
import java.security.PublicKey;

import org.apache.commons.pool.impl.GenericObjectPool;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * <p>
 * Description: Class used to encrypt and decrypt information using an
 * asymmetric algorithm.
 * </p>
 * <p>
 * This object is the responsible of the decryption of the information sent by
 * the client devices. The encryption algorithm used in this class must be
 * synchronized with the encryption algorithm used in the client device because
 * this class decrypts the information that the client devices sent encrypted
 * with an asymmetric algorithm
 * </p>
 * <p>
 * This class interacts with all the cryptographic objects using the Java
 * Cryptographic Extension (JCE). Using the JCE specification the change of the
 * cryptographic algorithm is done changing the name of the algorithms
 * </p>
 * <p>
 * This class always returns and expects printable strings. In the encryption
 * process after the codification of the information, the result byte array is
 * converted into a Base64 String. In the decryption process before START the
 * decryption proccess this class convert the Base64 String into a byte array
 * that is passed to the cipher to decrypt.
 * </p>
 * <p>
 * This class uses a pool to encrypt/decrypt the information received. The user
 * has to know that this class was designed to be created and stored a reference
 * to avoid the creation on the fly of ciphers. The creation of a assymmetric
 * cipher is a expensive operation. That's why this class creates a pool of
 * those objects and uses it when they are needed.
 * </p>
 * 
 * @version 1.0
 * @author Ithamar Torres
 */

public class AsymmetricCipherPool {

	/**
	 * This class maintains a pool of ciphers to allow concurrent encryption and
	 * decription of information. This field holds and manages the pool of
	 * instances used to encrypt/decrypt the information
	 */

	private GenericObjectPool cipherPool;

	/**
	 * Holds the pool size of the ciphers that this object will maintain to
	 * allow concurrent encryption/decryption process
	 */

	private static int POOL_SIZE = 10;

	/**
	 * Allows the user to initialize this object. It allows to set all the
	 * configurable elements that this class provides.
	 * 
	 * @param poolSize
	 *            The pool size of ciphers used in the decryption/decryption
	 *            process
	 */

	public static void init(int poolSize) {
		AsymmetricCipherPool.POOL_SIZE = poolSize;
	}

	/**
	 * <p>
	 * This constructor creates a AsymmetricCipher with the key received as
	 * argument used to encrypt/decrypt the information received by the server.
	 * </p>
	 * 
	 * @param key
	 *            PrivateKey Key used in the encryption/decryption process
	 * @param poolSize
	 *            The size of the cipher pool to create
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	public AsymmetricCipherPool(PrivateKey key, int poolSize) {
		this.cipherPool = new GenericObjectPool(
				new AsymmetricCipherPoolFactory(key));
		this.cipherPool.setMaxActive(poolSize);
		this.cipherPool.setMaxWait(poolSize);
		this.cipherPool.setMinIdle(poolSize);
		this.cipherPool
				.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
	}

	/**
	 * <p>
	 * This constructor creates a AsymmetricCipher with the key received as
	 * argument used to encrypt/decrypt the information received by the server.
	 * </p>
	 * 
	 * @param key
	 *            PrivateKey Key used in the encryption/decryption process
	 * @param poolSize
	 *            The size of the cipher pool to create
	 * @param maxWait
	 *            Max time to wait borrowing idle objects from the pool
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	public AsymmetricCipherPool(PrivateKey key, int poolSize, int maxWait) {
		this.cipherPool = new GenericObjectPool(
				new AsymmetricCipherPoolFactory(key));
		this.cipherPool.setMaxActive(poolSize);
		this.cipherPool.setMaxWait(maxWait);
		this.cipherPool.setMinIdle(poolSize);
		this.cipherPool
				.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
	}

	/**
	 * <p>
	 * This constructor creates a AsymmetricCipher with the key received as
	 * argument used to encrypt/decrypt the information received by the server.
	 * </p>
	 * 
	 * @param key
	 *            PrivateKey Key used in the encryption/decryption proccess
	 * @param poolSize
	 *            The size of the cipher pool to create
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	public AsymmetricCipherPool(PublicKey key, int poolSize) {
		this.cipherPool = new GenericObjectPool(
				new AsymmetricCipherPoolFactory(key));
		this.cipherPool.setMaxActive(poolSize);
		this.cipherPool.setMaxWait(poolSize);
		this.cipherPool.setMinIdle(poolSize);
		this.cipherPool
				.setWhenExhaustedAction(GenericObjectPool.WHEN_EXHAUSTED_BLOCK);
	}

	/**
	 * <p>
	 * This constructor creates a AsymmetricCipher with the private key received
	 * as argument used to encrypt/decrypt the information received by the
	 * server.
	 * </p>
	 * 
	 * @param privateKey
	 *            PrivateKey Key used in the encryption/decryption proccess
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	public AsymmetricCipherPool(PrivateKey privateKey) {
		this(privateKey, AsymmetricCipherPool.POOL_SIZE);
	}

	/**
	 * <p>
	 * This constructor creates a AsymmetricCipher with the private key received
	 * as argument used to encrypt/decrypt the information received by the
	 * server.
	 * </p>
	 * 
	 * @param publicKey
	 *            PublicKey Key used in the encryption/decryption process
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicKeyException
	 *             In case there is a problem with the key received as argument.
	 *             This exception could rise in case of an incorrect key length,
	 *             a corrupt key or any problem regarding the key used in the
	 *             algorithm.
	 */

	public AsymmetricCipherPool(PublicKey publicKey) {
		this(publicKey, AsymmetricCipherPool.POOL_SIZE);
	}

	/**
	 * <p>
	 * Method that decrypts a message encrypted with an asymmetric algorithm
	 * using the key received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            String The message that needs to be decrypted
	 * @return String The plain message.
	 * @throws CryptographicRuntimeException
	 *             An exception runtime occurred
	 */

	public String decrypt(String message) throws CryptographicRuntimeException {
		PoolableAsymmetricCipher cipher = null;
		try {
			cipher = (PoolableAsymmetricCipher) this.cipherPool.borrowObject();
			return cipher.decrypt(message);
		} catch (CryptographicRuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.CIPHER_POOL_EXTRACTION_PROBLEM,
					e);
		} finally {
			try {
				if (cipher != null) {
					this.cipherPool.returnObject(cipher);
				}
			} catch (Exception e) {

			}
		}
	}

	/**
	 * <p>
	 * Method that decrypts a message encrypted with an asymmetric algorithm
	 * using the key received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            byte[] The binary package that needs to be decrypted
	 * @return byte[] The binary plain message.
	 * @throws CryptographicRuntimeException
	 *             An exception runtime occurred
	 */

	public byte[] decryptBytes(byte[] message)
			throws CryptographicRuntimeException {
		PoolableAsymmetricCipher cipher = null;
		try {
			cipher = (PoolableAsymmetricCipher) this.cipherPool.borrowObject();
			return cipher.decryptBytes(message);
		} catch (CryptographicRuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.CIPHER_POOL_EXTRACTION_PROBLEM,
					e);
		} finally {
			try {
				if (cipher != null) {
					this.cipherPool.returnObject(cipher);
				}
			} catch (Exception e) {

			}
		}
	}

	/**
	 * <p>
	 * Method that encrypts a message using the key received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            String The message that needs to be encrypted
	 * @return String The encrypted message.
	 * @throws CryptographicRuntimeException
	 *             An exception runtime occurred
	 */

	public String encrypt(String message) throws CryptographicRuntimeException {
		PoolableAsymmetricCipher cipher = null;
		try {
			cipher = (PoolableAsymmetricCipher) this.cipherPool.borrowObject();
			return cipher.encrypt(message);
		} catch (CryptographicRuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.CIPHER_POOL_EXTRACTION_PROBLEM,
					e);
		} finally {
			try {
				if (cipher != null) {
					this.cipherPool.returnObject(cipher);
				}
			} catch (Exception e) {

			}
		}
	}

}
