/**
 * CertificateGenerator.java
 *
 * $Revision: 1.8 $ ($Date: 2011-04-06 04:47:29 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.asymmetric;

import java.math.BigInteger;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import java.util.Date;

import org.bouncycastle.asn1.x509.BasicConstraints;
import org.bouncycastle.asn1.x509.KeyUsage;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.jce.PrincipalUtil;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.x509.X509V3CertificateGenerator;
import org.bouncycastle.x509.extension.AuthorityKeyIdentifierStructure;

/**
 * X509 Certificate generator
 * 
 * @version 1.0 (03/07/2006)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class CertificateGenerator {

	/**
	 * Certificate common name label
	 */
	public static String COMMON_NAME_ID = "CN";

	/**
	 * Certificate organization unit identifier label
	 */
	private static String ORGANIZATION_UNIT_ID = "OU";

	/**
	 * Certificate organization identifier label
	 */
	private static String ORGANIZATION_ID = "O";

	/**
	 * Certificate location identifier label
	 */
	private static String LOCATION_ID = "L";

	/**
	 * Certificate state identifier label
	 */
	private static String STATE_ID = "ST";

	/**
	 * Certificate country identifier label
	 */
	private static String COUNTRY_ID = "C";

	/**
	 * Default certificate organization unit identifier value
	 */
	private static String ORGANIZATION_UNIT_VALUE = "SAES";

	/**
	 * Default certificate organization unit identifier value
	 */
	private static String ORGANIZATION_VALUE = "Smartmatic";

	/**
	 * Default certificate location value
	 */
	private static String LOCATION_VALUE = "Brussels";

	/**
	 * Default certificate state value
	 */
	private static String STATE_VALUE = "Brussels";

	/**
	 * Default certificate country value
	 */
	private static String COUNTRY_VALUE = "BE";

	/**
	 * Default certificate label key-value separator
	 */
	public static String SEPARATOR = "=";

	/**
	 * Default certificate labels separator
	 */
	private static String SPLITTER = ",";

	/**
	 * <p>
	 * This method is used to generate a certificate. It take the default values
	 * for organization, organization unit, location, state and country
	 * </p>
	 * 
	 * @param pubKey
	 *            The public key used to generate the certificate
	 * @param caKey
	 *            The private key used to generate the certificate
	 * @param caCert
	 *            The certificate for this certificate. If null is passed the
	 *            generated certificate is autocertificated
	 * @param serial
	 *            The serial number used to generate the certificate
	 * @param commonName
	 *            The common name used to generate the certificate
	 * @param startDate
	 *            The start date used to generate the certificate
	 * @param endDate
	 *            The end date used to generate the certificate
	 * @param signAlgorithm
	 *            Algorithm used to sign
	 * @return X509Certificate The generated certificate
	 * 
	 * @throws Exception
	 */
	public static X509Certificate generateCertificate(PublicKey pubKey,
			PrivateKey caKey, X509Certificate caCert, BigInteger serial,
			String commonName, Date startDate, Date endDate,
			String signAlgorithm) throws Exception {
		return CertificateGenerator.generateCertificate(pubKey, caKey, caCert,
				serial, commonName, startDate, endDate,
				CertificateGenerator.ORGANIZATION_UNIT_VALUE,
				CertificateGenerator.ORGANIZATION_VALUE,
				CertificateGenerator.LOCATION_VALUE,
				CertificateGenerator.STATE_VALUE,
				CertificateGenerator.COUNTRY_VALUE, signAlgorithm);
	}

	/**
	 * <p>
	 * This method is used to generate a certificate.
	 * </p>
	 * 
	 * @param pubKey
	 *            The public key used to generate the certificate
	 * @param caKey
	 *            The private key used to generate the certificate
	 * @param caCert
	 *            The certificate for this certificate. If null is passed the
	 *            generated certificate is autocertificated
	 * @param serial
	 *            The serial number used to generate the certificate
	 * @param commonName
	 *            The common name used to generate the certificate
	 * @param startDate
	 *            The start date used to generate the certificate
	 * @param endDate
	 *            The end date used to generate the certificate
	 * @param organizationUnitValue
	 *            The organization unit value used to generate the certificate
	 * @param organizationValue
	 *            The organization value used to generate the certificate
	 * @param location
	 *            The location used to generate the certificate
	 * @param state
	 *            The state used to generate the certificate
	 * @param country
	 *            The country used to generate the certificate
	 * @param signAlgorithm
	 *            Algorithm used to sign
	 * @return X509Certificate The generated certificate
	 * 
	 * @throws Exception
	 */

	public static X509Certificate generateCertificate(PublicKey pubKey,
			PrivateKey caKey, X509Certificate caCert, BigInteger serial,
			String commonName, Date startDate, Date endDate,
			String organizationUnitValue, String organizationValue,
			String location, String state, String country, String signAlgorithm)
			throws Exception {

		// load the certificate parameters
		X509V3CertificateGenerator generator = new X509V3CertificateGenerator();
		generator.setSerialNumber(serial);
		generator.setNotBefore(startDate);
		generator.setNotAfter(endDate);

		String organizationUnitStr = CertificateGenerator.ORGANIZATION_UNIT_ID
				+ CertificateGenerator.SEPARATOR + organizationUnitValue;
		String organizationStr = CertificateGenerator.ORGANIZATION_ID
				+ CertificateGenerator.SEPARATOR + organizationValue;
		String locationStr = CertificateGenerator.LOCATION_ID
				+ CertificateGenerator.SEPARATOR + location;
		String stateStr = CertificateGenerator.STATE_ID
				+ CertificateGenerator.SEPARATOR + state;
		String countryStr = CertificateGenerator.COUNTRY_ID
				+ CertificateGenerator.SEPARATOR + country;
		String commonNameStr = CertificateGenerator.COMMON_NAME_ID
				+ CertificateGenerator.SEPARATOR + commonName;

		String subjectStr = commonNameStr + CertificateGenerator.SPLITTER
				+ organizationUnitStr + CertificateGenerator.SPLITTER
				+ organizationStr + CertificateGenerator.SPLITTER + locationStr
				+ CertificateGenerator.SPLITTER + stateStr
				+ CertificateGenerator.SPLITTER + countryStr;

		if (caCert != null) {
			generator
					.setIssuerDN(PrincipalUtil.getSubjectX509Principal(caCert));
		} else {
			generator.setIssuerDN(new X509Name(commonNameStr
					+ CertificateGenerator.SPLITTER + organizationUnitStr));
		}

		generator.setSubjectDN(new X509Principal(subjectStr));
		generator.setPublicKey(pubKey);
		generator.setSignatureAlgorithm(signAlgorithm);

		// add the X509 extensions
		if (caCert != null) {
			generator.addExtension(X509Extensions.AuthorityKeyIdentifier,
					false, new AuthorityKeyIdentifierStructure(caCert));
		}

		// generator.addExtension(X509Extensions.SubjectKeyIdentifier, false,
		// new SubjectKeyIdentifierStructure(pubKey));
		generator.addExtension(X509Extensions.BasicConstraints, true,
				new BasicConstraints(false));
		generator.addExtension(X509Extensions.KeyUsage, true, new KeyUsage(
				KeyUsage.digitalSignature | KeyUsage.keyEncipherment));

		X509Certificate cert = generator.generateX509Certificate(caKey);
		return cert;
	}
}
