package com.smartmatic.waffle.security.sign;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.keyinfo.X509Data;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class SignDetachedGenerator {

	static {
		Security.insertProviderAt(
				new org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI(), 1);
	}

	/**
	 * Certificates used in the signing process
	 */

	private X509Certificate[] certificates;

	/**
	 * Private keys used in the signing process
	 */

	private PrivateKey[] privateKeys;

	/**
	 * Xml signature factory
	 */

	private XMLSignatureFactory xmlSignatureFactory;

	/**
	 * Signed info array
	 */

	private SignedInfo[] si;

	/**
	 * Key info array
	 */

	private KeyInfo[] ki;

	/**
	 * Set of base URIs used when there are more of than one base URI
	 */
	private Set<String> baseURISet;

	/**
	 * Algorithm to sign
	 */

	private String signatureAlgorithm;

	/**
	 * List of references to the files to be signed
	 */
	private List<Reference> fileReferences;

	/**
	 * Constructor. Creates a new sign generator instance.
	 * 
	 * @param certs
	 *            The array of certificates used in the signing process
	 * @param privateKeys
	 *            The private keys arrays used in the signing process
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidAlgorithmParameterException
	 */
	public SignDetachedGenerator(X509Certificate[] certs,
			PrivateKey[] privateKeys, String signatureAlgorithm)
			throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
		this.certificates = (certs != null) ? certs.clone() : null;
		this.privateKeys = (privateKeys != null) ? privateKeys.clone() : null;
		this.fileReferences = new ArrayList<Reference>();
		this.signatureAlgorithm = signatureAlgorithm;
		// Create a DOM XMLSignatureFactory that will be used to
		// generate the enveloped signature.
		this.xmlSignatureFactory = XMLSignatureFactory.getInstance("DOM");
	}

	/**
	 * Constructor. Creates a new sign generator instance with only a signing
	 * certificate and private key.
	 * 
	 * @param cert
	 *            The cert used in the signing process
	 * @param privateKeys
	 *            The private key used in the signing process
	 * 
	 * @throws NoSuchAlgorithmException
	 *             If a problem in the algorithms used in the signing process
	 *             occurs
	 * @throws InvalidAlgorithmParameterException
	 *             If a problem in one or more of the signing parameters occurs
	 */

	public SignDetachedGenerator(X509Certificate cert, PrivateKey privateKeys,
			String signatureAlgorithm) throws NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {
		this(new X509Certificate[] { cert }, new PrivateKey[] { privateKeys },
				signatureAlgorithm);
	}

	/**
	 * Adds references for a set of files
	 * 
	 * @param theBaseUri
	 *            The base URI of the files. The references will be relative to
	 *            this base URI.
	 * @param files
	 *            The files
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidAlgorithmParameterException
	 */
	public void addReferences(String theBaseUri, List<File> files)
			throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {

		if (this.baseURISet == null) {
			this.baseURISet = new HashSet<String>();
		}
		this.baseURISet.add(theBaseUri);

		// Create a Reference to the enveloped document, and also specify the
		// SHA256 digest algorithm and the ENVELOPED Transform.
		if (files != null && !files.isEmpty()) {
			for (File indexFile : files) {
				String relative = "";
				if (theBaseUri != null && !theBaseUri.isEmpty()) {
					relative = new File(theBaseUri).toURI()
							.relativize(indexFile.toURI()).getPath();
				}
				Reference ref = this.xmlSignatureFactory.newReference(relative,
						this.xmlSignatureFactory.newDigestMethod(
								DigestMethod.SHA256, null));
				this.fileReferences.add(ref);
			}
		}
	}

	/**
	 * Adds references for a set of files
	 * 
	 * @param theBaseUri
	 *            The base URI of the files. The references will be relative to
	 *            this base URI.
	 * @param files
	 *            The files
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidAlgorithmParameterException
	 */
	public void addReferencesFromCache(String theBaseUri,
			List<Map.Entry<File, byte[]>> files)
			throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {

		if (this.baseURISet == null) {
			this.baseURISet = new HashSet<String>();
		}
		this.baseURISet.add(theBaseUri);

		// Create a Reference to the enveloped document, and also specify the
		// SHA256 digest algorithm and the ENVELOPED Transform.
		if (files != null && !files.isEmpty()) {
			for (Map.Entry<File, byte[]> indexEntry : files) {
				String relative = "";
				if (theBaseUri != null && !theBaseUri.isEmpty()) {
					relative = new File(theBaseUri).toURI()
							.relativize(indexEntry.getKey().toURI()).getPath();
				}

				Reference ref;

				if (indexEntry.getValue() != null) {
					ref = this.xmlSignatureFactory.newReference(relative,
							this.xmlSignatureFactory.newDigestMethod(
									DigestMethod.SHA256, null), null, null,
							null, indexEntry.getValue());
				} else {
					ref = this.xmlSignatureFactory.newReference(relative,
							this.xmlSignatureFactory.newDigestMethod(
									DigestMethod.SHA256, null));
				}

				this.fileReferences.add(ref);
			}
		}
	}

	/**
	 * Adds the nodes SigInfo and KeyInfo to the signature document.
	 * 
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidAlgorithmParameterException
	 */
	private void addSignInfo() throws NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {

		if (this.fileReferences.isEmpty()) {
			Reference ref = this.xmlSignatureFactory.newReference("",
					this.xmlSignatureFactory.newDigestMethod(
							DigestMethod.SHA256, null));
			this.fileReferences.add(ref);
		}

		this.si = new SignedInfo[this.certificates.length];
		this.ki = new KeyInfo[this.certificates.length];

		SignatureMethod signatureMethod = this.xmlSignatureFactory
				.newSignatureMethod(this.signatureAlgorithm, null);

		CanonicalizationMethod canonicalizationMethod = this.xmlSignatureFactory
				.newCanonicalizationMethod(CanonicalizationMethod.INCLUSIVE,
						(C14NMethodParameterSpec) null);

		for (int i = 0; i < this.certificates.length; i++) {

			SignedInfo actualSi = this.xmlSignatureFactory.newSignedInfo(
					canonicalizationMethod, signatureMethod,
					this.fileReferences);

			// Create the KeyInfo containing the X509Data.
			KeyInfoFactory kif = this.xmlSignatureFactory.getKeyInfoFactory();
			List<Serializable> x509Content = new ArrayList<Serializable>();
			x509Content.add(this.certificates[i].getSubjectX500Principal()
					.getName());
			x509Content.add(this.certificates[i]);
			X509Data xd = kif.newX509Data(x509Content);
			KeyInfo actualKi = kif.newKeyInfo(Collections.singletonList(xd));

			this.si[i] = actualSi;
			this.ki[i] = actualKi;
		}
	}

	/**
	 * Signs and returns the signatures document.
	 * 
	 * @throws ParserConfigurationException
	 * @throws MarshalException
	 * @throws XMLSignatureException
	 * @throws InvalidAlgorithmParameterException
	 * @throws NoSuchAlgorithmException
	 */
	private Document getSignedDocument() throws ParserConfigurationException,
			MarshalException, XMLSignatureException, NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {
		// Add sign info
		addSignInfo();
		// Instantiate the document to be signed.
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		Document doc = dbf.newDocumentBuilder().newDocument();

		for (int i = 0; i < this.certificates.length; i++) {

			// Create a DOMSignContext and specify the RSA PrivateKey and
			// location of the resulting XMLSignature's parent element.
			DOMSignContext dsc = new DOMSignContext(this.privateKeys[i], doc);
			if (this.baseURISet != null && !this.baseURISet.isEmpty()) {
				dsc.setURIDereferencer(new DetachedURIDereferencer(
						this.baseURISet));
			} else {
				dsc.setBaseURI("file:///");
			}

			// Create the XMLSignature, but don't sign it yet.
			XMLSignature signature = this.xmlSignatureFactory.newXMLSignature(
					this.si[i], this.ki[i]);

			// Marshal, generate, and sign the enveloped signature.
			signature.sign(dsc);
		}
		return doc;
	}

	/**
	 * Signs and returns the contest of the signature document. The document
	 * will include the XML declaration.
	 * 
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws XMLSignatureException
	 * @throws MarshalException
	 * @throws TransformerException
	 * @throws InvalidAlgorithmParameterException
	 * @throws NoSuchAlgorithmException
	 */
	public String getSignedText() throws SAXException, IOException,
			ParserConfigurationException, MarshalException,
			XMLSignatureException, TransformerException,
			NoSuchAlgorithmException, InvalidAlgorithmParameterException {
		return getSignedText(true);
	}

	/**
	 * Signs and returns the contest of the signature document.
	 * 
	 * @param includeXMLDeclaration
	 *            Whether to include the XML declaration or not
	 * 
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws XMLSignatureException
	 * @throws MarshalException
	 * @throws TransformerException
	 * @throws InvalidAlgorithmParameterException
	 * @throws NoSuchAlgorithmException
	 */
	public String getSignedText(boolean includeXMLDeclaration)
			throws SAXException, IOException, ParserConfigurationException,
			MarshalException, XMLSignatureException, TransformerException,
			NoSuchAlgorithmException, InvalidAlgorithmParameterException {

		Document doc = this.getSignedDocument();
		// Output the resulting document.
		StreamResult result = new StreamResult(new StringWriter());
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer trans = tf.newTransformer();
		if (!includeXMLDeclaration) {
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		}
		trans.transform(new DOMSource(doc), result);
		String xmlString = result.getWriter().toString();
		return xmlString;
	}
}
